"""
GitLab API: https://docs.gitlab.com/ce/api/pipeline_schedules.html
"""
import pytest
import responses


@pytest.fixture
def resp_project_pipeline_schedule(created_content):
    content = {
        "id": 14,
        "description": "Build packages",
        "ref": "master",
        "cron": "0 1 * * 5",
        "cron_timezone": "UTC",
        "next_run_at": "2017-05-26T01:00:00.000Z",
        "active": True,
        "created_at": "2017-05-19T13:43:08.169Z",
        "updated_at": "2017-05-19T13:43:08.169Z",
        "last_pipeline": None,
        "owner": {
            "name": "Administrator",
            "username": "root",
            "id": 1,
            "state": "active",
            "avatar_url": "http://www.gravatar.com/avatar/e64c7d89f26bd1972efa854d13d7dd61?s=80&d=identicon",
            "web_url": "https://gitlab.example.com/root",
        },
    }

    with responses.RequestsMock() as rsps:
        rsps.add(
            method=responses.POST,
            url="http://localhost/api/v4/projects/1/pipeline_schedules",
            json=content,
            content_type="application/json",
            status=200,
        )
        rsps.add(
            method=responses.POST,
            url="http://localhost/api/v4/projects/1/pipeline_schedules/14/play",
            json=created_content,
            content_type="application/json",
            status=201,
        )
        yield rsps


def test_project_pipeline_schedule_play(project, resp_project_pipeline_schedule):
    description = "Build packages"
    cronline = "0 1 * * 5"
    sched = project.pipelineschedules.create(
        {"ref": "master", "description": description, "cron": cronline}
    )
    assert sched is not None
    assert description == sched.description
    assert cronline == sched.cron

    play_result = sched.play()
    assert play_result is not None
    assert "message" in play_result
    assert play_result["message"] == "201 Created"



"""
GitLab API: https://docs.gitlab.com/ce/api/issues.html
"""

import pytest
import responses

from gitlab.v4.objects import ProjectIssuesStatistics


@pytest.fixture
def resp_issue():
    content = [{"name": "name", "id": 1}, {"name": "other_name", "id": 2}]

    with responses.RequestsMock() as rsps:
        rsps.add(
            method=responses.GET,
            url="http://localhost/api/v4/issues",
            json=content,
            content_type="application/json",
            status=200,
        )
        yield rsps


@pytest.fixture
def resp_issue_statistics():
    content = {"statistics": {"counts": {"all": 20, "closed": 5, "opened": 15}}}

    with responses.RequestsMock() as rsps:
        rsps.add(
            method=responses.GET,
            url="http://localhost/api/v4/projects/1/issues_statistics",
            json=content,
            content_type="application/json",
            status=200,
        )
        yield rsps


def test_issues(gl, resp_issue):
    data = gl.issues.list()
    assert data[1].id == 2
    assert data[1].name == "other_name"


def test_project_issues_statistics(project, resp_issue_statistics):
    statistics = project.issuesstatistics.get()
    assert isinstance(statistics, ProjectIssuesStatistics)
    assert statistics.statistics["counts"]["all"] == 20



#!/usr/bin/env python

import sys

from gitlab import Gitlab


def main():
    with Gitlab.from_config(config_files=["/tmp/python-gitlab.cfg"]) as gl:
        for project in gl.projects.list():
            project.delete()
        for group in gl.groups.list():
            group.delete()
        for user in gl.users.list():
            if user.username != "root":
                user.delete()


if __name__ == "__main__":
    sys.exit(main())



"""
GitLab API: https://docs.gitlab.com/ce/api/project_import_export.html
"""
import pytest
import responses


@pytest.fixture
def resp_import_project():
    content = {
        "id": 1,
        "description": None,
        "name": "api-project",
        "name_with_namespace": "Administrator / api-project",
        "path": "api-project",
        "path_with_namespace": "root/api-project",
        "created_at": "2018-02-13T09:05:58.023Z",
        "import_status": "scheduled",
    }

    with responses.RequestsMock() as rsps:
        rsps.add(
            method=responses.POST,
            url="http://localhost/api/v4/projects/import",
            json=content,
            content_type="application/json",
            status=200,
        )
        yield rsps


@pytest.fixture
def resp_import_status():
    content = {
        "id": 1,
        "description": "Itaque perspiciatis minima aspernatur corporis consequatur.",
        "name": "Gitlab Test",
        "name_with_namespace": "Gitlab Org / Gitlab Test",
        "path": "gitlab-test",
        "path_with_namespace": "gitlab-org/gitlab-test",
        "created_at": "2017-08-29T04:36:44.383Z",
        "import_status": "finished",
    }

    with responses.RequestsMock() as rsps:
        rsps.add(
            method=responses.GET,
            url="http://localhost/api/v4/projects/1/import",
            json=content,
            content_type="application/json",
            status=200,
        )
        yield rsps


@pytest.fixture
def resp_import_github():
    content = {
        "id": 27,
        "name": "my-repo",
        "full_path": "/root/my-repo",
        "full_name": "Administrator / my-repo",
    }

    with responses.RequestsMock() as rsps:
        rsps.add(
            method=responses.POST,
            url="http://localhost/api/v4/import/github",
            json=content,
            content_type="application/json",
            status=200,
        )
        yield rsps


def test_import_project(gl, resp_import_project):
    project_import = gl.projects.import_project("file", "api-project")
    assert project_import["import_status"] == "scheduled"


def test_refresh_project_import_status(project, resp_import_status):
    project_import = project.imports.get()
    project_import.refresh()
    assert project_import.import_status == "finished"


def test_import_github(gl, resp_import_github):
    base_path = "/root"
    name = "my-repo"
    ret = gl.projects.import_github("githubkey", 1234, base_path, name)
    assert isinstance(ret, dict)
    assert ret["name"] == name
    assert ret["full_path"] == "/".join((base_path, name))
    assert ret["full_name"].endswith(name)


def test_create_project_export(project, resp_export):
    export = project.exports.create()
    assert export.message == "202 Accepted"


def test_refresh_project_export_status(project, resp_export):
    export = project.exports.create()
    export.refresh()
    assert export.export_status == "finished"


def test_download_project_export(project, resp_export, binary_content):
    export = project.exports.create()
    download = export.download()
    assert isinstance(download, bytes)
    assert download == binary_content



import pytest

from gitlab import exceptions


def test_error_raises_from_http_error():
    """Methods decorated with @on_http_error should raise from GitlabHttpError."""

    class TestError(Exception):
        pass

    @exceptions.on_http_error(TestError)
    def raise_error_from_http_error():
        raise exceptions.GitlabHttpError

    with pytest.raises(TestError) as context:
        raise_error_from_http_error()
    assert isinstance(context.value.__cause__, exceptions.GitlabHttpError)



"""
GitLab API: https://docs.gitlab.com/ce/api/remote_mirrors.html
"""

import pytest
import responses

from gitlab.v4.objects import ProjectRemoteMirror


@pytest.fixture
def resp_remote_mirrors():
    content = {
        "enabled": True,
        "id": 1,
        "last_error": None,
        "last_successful_update_at": "2020-01-06T17:32:02.823Z",
        "last_update_at": "2020-01-06T17:32:02.823Z",
        "last_update_started_at": "2020-01-06T17:31:55.864Z",
        "only_protected_branches": True,
        "update_status": "none",
        "url": "https://*****:*****@gitlab.com/gitlab-org/security/gitlab.git",
    }

    with responses.RequestsMock(assert_all_requests_are_fired=False) as rsps:
        rsps.add(
            method=responses.GET,
            url="http://localhost/api/v4/projects/1/remote_mirrors",
            json=[content],
            content_type="application/json",
            status=200,
        )
        rsps.add(
            method=responses.POST,
            url="http://localhost/api/v4/projects/1/remote_mirrors",
            json=content,
            content_type="application/json",
            status=200,
        )

        updated_content = dict(content)
        updated_content["update_status"] = "finished"

        rsps.add(
            method=responses.PUT,
            url="http://localhost/api/v4/projects/1/remote_mirrors/1",
            json=updated_content,
            content_type="application/json",
            status=200,
        )
        yield rsps


def test_list_project_remote_mirrors(project, resp_remote_mirrors):
    mirrors = project.remote_mirrors.list()
    assert isinstance(mirrors, list)
    assert isinstance(mirrors[0], ProjectRemoteMirror)
    assert mirrors[0].enabled


def test_create_project_remote_mirror(project, resp_remote_mirrors):
    mirror = project.remote_mirrors.create({"url": "https://example.com"})
    assert isinstance(mirror, ProjectRemoteMirror)
    assert mirror.update_status == "none"


def test_update_project_remote_mirror(project, resp_remote_mirrors):
    mirror = project.remote_mirrors.create({"url": "https://example.com"})
    mirror.only_protected_branches = True
    mirror.save()
    assert mirror.update_status == "finished"
    assert mirror.only_protected_branches
